@@ -14,6 +14,7 @@ use solana_sdk::{
     commitment_config::CommitmentConfig,
     signature::{Keypair, Signer},
 };
+use solana_streamer::socket::SocketAddrSpace;
 use solana_vote_program::vote_state::{VoteAuthorize, VoteState, VoteStateVersions};
 
 #[test]
@@ -21,7 +22,8 @@ fn test_vote_authorize_and_withdraw() {
     let mint_keypair = Keypair::new();
     let mint_pubkey = mint_keypair.pubkey();
     let faucet_addr = run_local_faucet(mint_keypair, None);
-    let test_validator = TestValidator::with_no_fees(mint_pubkey, Some(faucet_addr));
+    let test_validator =
+        TestValidator::with_no_fees(mint_pubkey, Some(faucet_addr), SocketAddrSpace::Unspecified);
 
     let rpc_client =
         RpcClient::new_with_commitment(test_validator.rpc_url(), CommitmentConfig::processed());
@@ -83,13 +85,48 @@ fn test_vote_authorize_and_withdraw() {
     check_recent_balance(expected_balance, &rpc_client, &vote_account_pubkey);
 
     // Authorize vote account withdrawal to another signer
-    let withdraw_authority = Keypair::new();
+    let first_withdraw_authority = Keypair::new();
     config.signers = vec![&default_signer];
+    config.command = CliCommand::VoteAuthorize {
+        vote_account_pubkey,
+        new_authorized_pubkey: first_withdraw_authority.pubkey(),
+        vote_authorize: VoteAuthorize::Withdrawer,
+        memo: None,
+        authorized: 0,
+        new_authorized: None,
+    };
+    process_command(&config).unwrap();
+    let vote_account = rpc_client
+        .get_account(&vote_account_keypair.pubkey())
+        .unwrap();
+    let vote_state: VoteStateVersions = vote_account.state().unwrap();
+    let authorized_withdrawer = vote_state.convert_to_current().authorized_withdrawer;
+    assert_eq!(authorized_withdrawer, first_withdraw_authority.pubkey());
+
+    // Authorize vote account withdrawal to another signer with checked instruction
+    let withdraw_authority = Keypair::new();
+    config.signers = vec![&default_signer, &first_withdraw_authority];
     config.command = CliCommand::VoteAuthorize {
         vote_account_pubkey,
         new_authorized_pubkey: withdraw_authority.pubkey(),
         vote_authorize: VoteAuthorize::Withdrawer,
         memo: None,
+        authorized: 1,
+        new_authorized: Some(1),
+    };
+    process_command(&config).unwrap_err(); // unsigned by new authority should fail
+    config.signers = vec![
+        &default_signer,
+        &first_withdraw_authority,
+        &withdraw_authority,
+    ];
+    config.command = CliCommand::VoteAuthorize {
+        vote_account_pubkey,
+        new_authorized_pubkey: withdraw_authority.pubkey(),
+        vote_authorize: VoteAuthorize::Withdrawer,
+        memo: None,
+        authorized: 1,
+        new_authorized: Some(2),
     };
     process_command(&config).unwrap();
     let vote_account = rpc_client
